package es.verdnatura.presentation.base

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.annotation.LayoutRes
import androidx.databinding.DataBindingUtil
import androidx.databinding.ViewDataBinding
import androidx.fragment.app.Fragment
import org.koin.androidx.viewmodel.ext.android.viewModel
import kotlin.reflect.KClass

abstract class BaseFragment<T : ViewDataBinding, V : BaseViewModel>(viewModelClass: KClass<V>) :
    Fragment() {


    protected val PREFS_USER = "es.verdnatura.user.prefs"
    protected val USERFK = "userFk"
    protected val USER = "user"
    protected val PASSWORD = "password"
    protected val TOKEN = "token"
    protected val SECTORDESCRIP = "sectordescrip"
    protected val SECTORFK = "sectorFk"
    protected val WAREHOUSEFK = "warehouseFk"
    protected val viewModel: V by viewModel(viewModelClass)
    protected lateinit var binding: T


    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? = inflater.inflate(getLayoutId(), container, false)

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)

        initDataBinding()
        getBundleArguments()
        observeViewModel()

        init()
    }

    @LayoutRes
    abstract fun getLayoutId(): Int
    open fun init() {}
    open fun observeViewModel() {}
    open fun getBundleArguments() {}
    open fun addBindingVariables() {}


    private fun initDataBinding() {
        binding = DataBindingUtil.bind<T>(view!!)!!
        binding.lifecycleOwner = viewLifecycleOwner
        binding.setVariable(BR.viewModel, this@BaseFragment.viewModel)
        lifecycle.addObserver(this@BaseFragment.viewModel)
        addBindingVariables()
    }
}
