package es.verdnatura.presentation.common

import android.content.Context
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentActivity
import androidx.fragment.app.FragmentManager
import androidx.fragment.app.FragmentTransaction
import com.bumptech.glide.Glide
import com.bumptech.glide.request.target.CustomTarget
import com.bumptech.glide.request.transition.Transition


inline fun FragmentManager.inTransaction(func: FragmentTransaction.() -> FragmentTransaction) =
    beginTransaction().func().commit()


fun <T> Context.openActivity(it: Class<T>, extras: Bundle.() -> Unit = {}) {
    val intent = Intent(this, it)
    intent.putExtras(Bundle().apply(extras))
    startActivity(intent)
}

fun FragmentActivity.replaceFragment(
    fragment: Fragment,
    frameId: Int,
    tag: String,
    addToBackStack: Boolean = false
) {
    supportFragmentManager.inTransaction {
        replace(frameId, fragment, tag)
        if (addToBackStack) addToBackStack(tag)
        setCustomAnimations(android.R.anim.fade_in, android.R.anim.fade_out)
    }
}

fun FragmentActivity.addFragment(
    fragment: Fragment,
    frameId: Int,
    tag: String = "",
    addToBackStack: Boolean = false
) {
    supportFragmentManager.inTransaction {
        add(frameId, fragment, tag)
        if (addToBackStack) addToBackStack(tag)
        setCustomAnimations(android.R.anim.fade_in, android.R.anim.fade_out)
    }
}

fun View.setDimensions(width: Int, height: Int) {
    val layoutParams = this.layoutParams
    layoutParams?.width = width
    layoutParams?.height = height
    this.layoutParams = layoutParams
}
fun View.setMargins(left: Int, top: Int, right: Int, bottom: Int) {
    val mlp = this.layoutParams as ViewGroup.MarginLayoutParams
    mlp.setMargins(left, top, right, bottom)
}

fun ImageView.loadUrl(url: String) {
//    load(url)
//    Glide.with(context).load(url).into(this)
    Glide.with(context)
        .asBitmap()
       .load(url)
       .into(object : CustomTarget<Bitmap>() {

           override fun onLoadCleared(placeholder: Drawable?) {}

           override fun onResourceReady(resource: Bitmap, transition: Transition<in Bitmap>?) {
                setImageBitmap(resource)
            }
        })
}

inline val <reified T> T.TAG: String
    get() = T::class.java.canonicalName ?: T::class.simpleName ?: T::class.java.simpleName