package es.verdnatura.presentation.view.feature.ajustes.fragment

import android.app.AlertDialog
import android.content.SharedPreferences
import android.os.Bundle
import android.view.View
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentAjustesBinding
import es.verdnatura.domain.notNull
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnAjustesItemClickListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.ajustes.adapter.AjustesAdapter
import es.verdnatura.presentation.view.feature.ajustes.model.AjustesItemVO
import es.verdnatura.presentation.view.feature.ajustes.model.SectorItemVO
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_ajustes.*

class AjustesFragment : BaseFragment<FragmentAjustesBinding,AjustesViewModel>(AjustesViewModel::class) {

    private var user:String? = ""
    private var password:String? = ""
    private lateinit var customDialog: CustomDialog
    private var sectorListVO:List<SectorItemVO> = listOf()
    private var prefs: SharedPreferences? = null
    private var ajustesAdapter: AjustesAdapter? = null

    companion object {
        fun newInstance() = AjustesFragment()
    }
    override fun getLayoutId(): Int = R.layout.fragment_ajustes

    override fun onCreate(savedInstanceState: Bundle?) {
        prefs = activity!!.getSharedPreferences(PREFS_USER,0)
        customDialog = CustomDialog(requireContext())
        viewModel.inititializeDefaultAjusts(prefs!!.getString(SECTORDESCRIP,"Sin sector").toString(),prefs!!.getInt(SECTORFK,0),prefs!!.getInt(WAREHOUSEFK,0))

        super.onCreate(savedInstanceState)
    }

    override fun init() {
        activity!!.main_bottom_navigation.visibility = View.VISIBLE
        user_text.setText(prefs!!.getString(USERFK,""))
        val versionName = activity!!.packageManager.getPackageInfo(activity!!.packageName,0).versionName
        item_version.setText(versionName)
        user = prefs!!.getString(USER,"")
        password = prefs!!.getString(PASSWORD,"")
        super.init()
    }


    override fun observeViewModel() {
        with(viewModel){
            ajustesAdapter = AjustesAdapter(ajustesitem,object: OnAjustesItemClickListener{
                override fun onAjustesItemClickListener(item: AjustesItemVO) {
                    if (item.id == 0){
                        splash_progress.visibility = View.VISIBLE
                        viewModel.getSectors(user!!,password!!)
                    } else if (item.id == 1){
                        activity!!.onBackPressed()
                    }
                }
            })
            ajustess_items.adapter = ajustesAdapter
            ajustess_items.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

            loadSectorList.observe(viewLifecycleOwner, Observer { event ->
                event.getContentIfNotHandled().notNull {
                    splash_progress.visibility = View.INVISIBLE
                    if (it.list != null && it.list.size > 0 && it.list.get(0).isError){
                        customDialog.setTitle("Error").setDescription(it.list.get(0).errorMessage).setOkButton("Cerrar"){
                            customDialog.hide()
                        }.show()
                    }else{

                        if (it.list != null && it.list.size > 0){
                            val listSectores : ArrayList<String> = ArrayList()
                            it.list.forEach {
                                listSectores.add(it.description)
                            }
                            val array = arrayOfNulls<String>(listSectores.size)
                            sectorListVO = it.list
                            showDialogSector(listSectores.toArray(array))

                        }else{
                            customDialog.setTitle("Sectores").setDescription("No existes sectores.").setOkButton("Cerrar"){
                                customDialog.hide()
                            }.show()
                        }

                    }
                }
            })
        }
        super.observeViewModel()
    }

    private fun showDialogSector(array:Array<String>){
        val builder = AlertDialog.Builder(this.context)


        builder.setTitle("Selecciona un sector")
        builder.setItems(array) { _, which ->
            val selected = array[which]
            sectorListVO.forEach {
                if (it.description.equals(selected)){
                    val editor = prefs!!.edit()
                    editor.putString(SECTORDESCRIP,it.description)
                    editor.putInt(SECTORFK,it.id)
                    it.warehouseFk?.let { it1 -> editor.putInt(WAREHOUSEFK, it1) }
                    editor.apply()

                    viewModel.ajustesitem.get(0).sectorFk = it.id
                    viewModel.ajustesitem.get(0).warehouse = it.warehouseFk
                    viewModel.ajustesitem.get(0).selected = it.description
                    ajustesAdapter!!.notifyDataSetChanged()

                    return@forEach
                }
            }


        }

        val dialog = builder.create()
        dialog.show()
    }




}
