package es.verdnatura.presentation.view.feature.ajustes.fragment


import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations
import es.verdnatura.domain.GetAjustesUserCase
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.view.feature.ajustes.model.AjustesItemVO
import es.verdnatura.presentation.view.feature.ajustes.model.SectorItemVO
import es.verdnatura.presentation.view.feature.ajustes.model.SectorListVO
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response


class AjustesViewModel : BaseViewModel() {

     private val getAjustesUserCase:GetAjustesUserCase = GetAjustesUserCase()
     val version : String = "5.0.0";

     private val _ajustesitem by lazy { ArrayList<AjustesItemVO>() }
     val ajustesitem: List<AjustesItemVO>
          get() = _ajustesitem

     private val _sectorList by lazy { MutableLiveData<SectorListVO>() }
     val sectorList: LiveData<SectorListVO>
          get() = _sectorList


     val loadSectorList = Transformations.map(_sectorList) {Event(it)}




     fun inititializeDefaultAjusts(sectorDescrip: String,sectorFk : Int, warehouseFk : Int) {
          _ajustesitem.add(
               AjustesItemVO(0,
                    "Sector",
                    sectorDescrip,
                    sectorFk,
                    warehouseFk)
          )

          _ajustesitem.add(
               AjustesItemVO(1,
                    "Cerrar Sesión",
                    "",
                    0,
                    0)
          )

     }

     fun getSectors(user:String,password:String){
          getAjustesUserCase.getSectors(user,password).enqueue(object : Callback<List<SectorItemVO>>{
               override fun onFailure(call: Call<List<SectorItemVO>>, t: Throwable) {
                    val listError:ArrayList<SectorItemVO> = ArrayList()
                    listError.add(SectorItemVO(0,"",0,true,t.message!!))
                    _sectorList.value = SectorListVO(listError)
               }

               override fun onResponse(
                    call: Call<List<SectorItemVO>>,
                    response: Response<List<SectorItemVO>>
               ) {
                    if (response.body() != null){
                         _sectorList.value = response.body()?.let { SectorListVO(it) }
                    }else{
                         val listError:ArrayList<SectorItemVO> = ArrayList()
                         listError.add(SectorItemVO(0,"",0,true,"Error en la llamada sector_get"))
                         _sectorList.value = SectorListVO(listError)
                    }
               }

          })
     }

}
