package es.verdnatura.presentation.view.feature.articulo.fragment



import android.content.Intent
import android.content.SharedPreferences
import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentItemCardBinding
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnBarcodeRowClickListener
import es.verdnatura.presentation.common.OnItemCardRowClickListener
import es.verdnatura.presentation.common.loadUrl
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.component.CustomDialogList
import es.verdnatura.presentation.view.component.CustomDialogTwoButtons
import es.verdnatura.presentation.view.feature.articulo.adapter.BarcodeAdapter
import es.verdnatura.presentation.view.feature.articulo.adapter.ItemCardAdapter
import es.verdnatura.presentation.view.feature.articulo.model.BarcodeVO
import es.verdnatura.presentation.view.feature.articulo.model.ItemCardRowVO
import es.verdnatura.presentation.view.feature.articulo.model.ItemCardVO
import es.verdnatura.presentation.view.feature.imageview.activity.ImageViewActivity
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_item_card.*
import kotlinx.android.synthetic.main.toolbar.*

class ItemCardFragment(
    var itemFk:String = ""
) : BaseFragment<FragmentItemCardBinding,ItemCardViewModel>(ItemCardViewModel::class) {

    private var urlLarge:String = ""
    private var titleImage:String = ""
    private var user = ""
    private var password = ""
    private var warehouseFk = ""
    private var adapter : ItemCardAdapter? = null
    private var barcodeAdapter : BarcodeAdapter? = null
    private var listItemsRow:ArrayList<ItemCardRowVO> = ArrayList()
    private var listBarcodes:ArrayList<BarcodeVO> = ArrayList()

    companion object {
        fun newInstance(entryPoint:String) = ItemCardFragment(entryPoint)
    }
    private lateinit var customDialog: CustomDialog
    private lateinit var customDialogInput: CustomDialogInput
    private lateinit var customDialogList: CustomDialogList
    private lateinit var customDialogTwo: CustomDialogTwoButtons

    override fun getLayoutId(): Int = R.layout.fragment_item_card

    override fun init() {
        itemcard_layout.visibility = View.GONE
        activity!!.main_bottom_navigation.visibility = View.GONE
        setEvents()
        toolbar_title.text = "ItemCard"
        customDialog = CustomDialog(requireContext())
        customDialogInput = CustomDialogInput(requireContext())
        customDialogList = CustomDialogList(requireContext())
        customDialogTwo = CustomDialogTwoButtons(requireContext())
        if (!itemFk.isNullOrEmpty()){
            getItemCard(itemFk)
        }
        super.init()
    }


    private fun setEvents(){
        edit_itemFk.requestFocus()
        edit_itemFk.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!edit_itemFk.text.toString().isNullOrEmpty())
                    getItemCard(edit_itemFk.text.toString())
                edit_itemFk.setText("")
                (activity as MainActivity).hideKeyboard(edit_itemFk)
                return@setOnEditorActionListener true
            }
            false
        }


        backButton.setOnClickListener {
            activity!!.onBackPressed()
        }

        itemcard_image.setOnClickListener {
            val i:Intent = Intent(activity,ImageViewActivity::class.java)
            i.putExtra("url",urlLarge)
            i.putExtra("title",titleImage)
            startActivity(i)
        }
    }

    private fun getItemCard(itemFk:String){
        val prefs: SharedPreferences = activity!!.getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        this.itemFk = itemFk
        splash_progress.visibility = View.VISIBLE
        viewModel.getItemCard(itemFk,warehouseFk,user!!,password!!)
    }

    override fun observeViewModel() {
        with(viewModel){
            itemcard.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                if (it.isError){
                    itemcard_layout.visibility = View.GONE
                    toolbar_title.text = "ItemCard"
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.hide()
                    }.show()
                }else{
                    if (it.id != "0"){
                        itemcard_layout.visibility = View.VISIBLE
                        setItemCard(it)
                    }else{
                        itemcard_layout.visibility = View.GONE
                        toolbar_title.text = "ItemCard"
                        customDialog.setTitle("Sin resultados").setDescription("No hemos podido encontrar el articulo. Revisa el sector.").setOkButton("Cerrar"){
                            customDialog.hide()
                        }.show()
                    }

                }
            })

            response.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.GONE
                if (it.isError){
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.hide()
                    }.show()
                }
            })
        }
    }

    private fun setItemCard(itemInfo:ItemCardVO){
        toolbar_title.text = itemInfo.id+"-"+itemInfo.longName
        titleImage = itemInfo.id+"-"+itemInfo.longName
        itemcard_image.setImageDrawable(resources.getDrawable(R.drawable.loadphoto,resources.newTheme()))
        itemcard_image.loadUrl(itemInfo.urlImage200)
        urlLarge = itemInfo.urlImage
        itemcard_tag1.text = itemInfo.size
        itemcard_tag2.text = itemInfo.value5
        itemcard_tag3.text = itemInfo.value6
        itemcard_tag4.text = itemInfo.value7 + " " + itemInfo.origin
        listItemsRow = ArrayList()
        listItemsRow.add(ItemCardRowVO(title = "Total",value = itemInfo.total,isEditable = false))
        listItemsRow.add(ItemCardRowVO(title = "Disponible",value = itemInfo.available,isEditable = false))
        listItemsRow.add(ItemCardRowVO(title = "Ubicado",value = itemInfo.enAltillo,isEditable = false))

        //EDITABLES
        listItemsRow.add(ItemCardRowVO(title = "SIN UBICAR",value = itemInfo.enNicho,isEditable = true, action = "itemStockUpdate"))
        listItemsRow.add(ItemCardRowVO(title = "NICHO",value = itemInfo.nicho,isEditable = true, action = "itemPlacementSave"))
        listItemsRow.add(ItemCardRowVO(title = "RESERVA",value = itemInfo.reserva,isEditable = true, action = "itemPlacementSave"))
        listItemsRow.add(ItemCardRowVO(title = "GROUPING",value = itemInfo.grouping,isEditable = true, action = "updateGrouping"))
        listItemsRow.add(ItemCardRowVO(title = "PACKING",value = itemInfo.packing,isEditable = true, action = "updatePacking"))
        listItemsRow.add(ItemCardRowVO(title = "MINIMO",value = itemInfo.min,isEditable = true, action = "itemSaveMin"))
        listItemsRow.add(ItemCardRowVO(title = "BARCODE",barcodes = itemInfo.barcodes,isEditable = true, action = "toBarcode"))
        listBarcodes = itemInfo.barcodes as ArrayList<BarcodeVO>

        adapter = ItemCardAdapter(listItemsRow,object: OnItemCardRowClickListener{
            override fun onItemCardRowClickListener(item: ItemCardRowVO) {


                if(item.action != "toBarcode"){
                    customDialogInput.setTitle(item.title!!).setDescription("Valor actual: "+item.value!!).setOkButton("Guardar"){
                        (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())
                        editItemCardRow(item,customDialogInput.getValue())
                        customDialogInput.hide()

                    }.setKoButton("Cancelar"){
                        (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())
                        customDialogInput.hide()
                    }.setValue("").show()
                    customDialogInput.getEditText().requestFocus()
                    customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
                        if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                            if (!customDialogInput.getValue().isNullOrEmpty())
                                editItemCardRow(item,customDialogInput.getValue())
                            customDialogInput.setValue("")
                            customDialogInput.hide()
                            (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())
                            return@setOnEditorActionListener true
                        }
                        false
                    }
                }else if(item.action == "toBarcode"){
                    //BARCODE ROW
                    prepareBarcodeDialog(item)
                }

            }
        })
        itemcard_recyclerview.adapter = adapter
        itemcard_recyclerview.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

        scroll_view.post(Runnable {
            val position:IntArray = IntArray(2)
            itemcard_image.getLocationInWindow(position)
            scroll_view.scrollTo(0, position[1])

        })

    }

    private fun editItemCardRow(item:ItemCardRowVO,value:String){

        when (item.action){
            "itemStockUpdate" -> prepareItemStockUpdate(item,value)
            "itemPlacementSave" -> viewModel.itemPlacementSave(itemFk = itemFk, warehouseFk = warehouseFk, user = user, password = password, value = value)
            "updateGrouping" -> viewModel.updateGrouping(itemFk = itemFk, user = user, password = password, value = value,warehouseFk = warehouseFk)
            "updatePacking" -> viewModel.updatePacking(itemFk = itemFk, user = user, password = password, value = value,warehouseFk = warehouseFk)
            "itemSaveMin" -> viewModel.itemSaveMin(itemFk = itemFk, user = user, password = password, value = value)
        }
        if(item.action != "itemStockUpdate")
            changeOfflineValue(item,value, listBarcodes)
    }

    private fun changeOfflineValue(item:ItemCardRowVO,newValue:String, barcodes:List<BarcodeVO>){
        var i = 0
        var position = 0
        listItemsRow.forEach {
            if (it.title == item.title) {
                position = i
            }
            i = i.plus(1)
        }
        if (listItemsRow.get(position).title != "BARCODE") listItemsRow.get(position).value = newValue
        else listItemsRow.get(position).barcodes = barcodes
        adapter?.notifyDataSetChanged()
    }

    private fun prepareItemStockUpdate(itemB:ItemCardRowVO,value:String){
        if (itemB.value!!.toInt() > value.toInt()) {
            customDialogTwo.setTitle(itemB.title!!).setDescription("Indica la causa de eliminar stock:").setOkButton("Falta"){
                viewModel.itemStockUpdate(itemFk = itemFk,warehouseFk = warehouseFk,user = user,password = password,newValue = value,isTrash = "0")
                changeOfflineValue(itemB,value, listBarcodes)
                customDialogTwo.hide()

            }.setOkButtonTwo("Basura"){
                viewModel.itemStockUpdate(itemFk = itemFk,warehouseFk = warehouseFk,user = user,password = password,newValue = value,isTrash = "1")
                changeOfflineValue(itemB,value, listBarcodes)
                customDialogTwo.hide()

            }.setKoButton("Cancelar"){

                customDialogTwo.hide()
            }.show()
        }else{
            changeOfflineValue(itemB,value, listBarcodes)
            viewModel.itemStockUpdate(itemFk = itemFk,warehouseFk = warehouseFk,user = user,password = password,newValue = value,isTrash = "0")
        }


    }

    private fun prepareBarcodeDialog(itemB:ItemCardRowVO){

        customDialogList.setTitle("Barcodes").setOkButton("Guardar"){
            (activity as MainActivity).hideKeyboard(customDialogList.getEditText())
            if (!customDialogList.getValue().isNullOrEmpty())
                updateBarcode(customDialogList.getValue(),"0",itemB)
            customDialogList.hide()

        }.setKoButton("Cerrar"){
            (activity as MainActivity).hideKeyboard(customDialogList.getEditText())
            customDialogList.hide()
        }.setValue("").show()


        (activity as MainActivity).hideKeyboard(customDialogList.getEditText())
        customDialogList.getEditText().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                if (!customDialogList.getValue().isNullOrEmpty())
                    updateBarcode(customDialogList.getValue(),"0",itemB)
                customDialogList.setValue("")
                customDialogList.hide()
                (activity as MainActivity).hideKeyboard(customDialogList.getEditText())
                return@setOnEditorActionListener true
            }
            false
        }

        barcodeAdapter = BarcodeAdapter(listBarcodes,object: OnBarcodeRowClickListener{
            override fun onBarcodeRowClickListener(item: BarcodeVO) {
                updateBarcode(item.code!!,"1",itemB)
            }
        })
        customDialogList.getRecyclerView().adapter = barcodeAdapter

        customDialogList.getRecyclerView().layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
    }

    private fun updateBarcode(code:String,delete:String,item:ItemCardRowVO){
        if (delete == "1"){
            var i = 0
            var posDelete = 0
            listBarcodes.forEach {
                if (it.code == code)
                    posDelete = i
                i+=1
            }
            listBarcodes.removeAt(posDelete)
        }else{
            listBarcodes.add(BarcodeVO(code))
        }
        viewModel.barcodesEdit(itemFk = itemFk, user = user, password = password, value = code, delete = delete)
        barcodeAdapter!!.notifyDataSetChanged()
        changeOfflineValue(item,"", listBarcodes)
    }

}
