package es.verdnatura.presentation.view.feature.articulo.fragment


import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import es.verdnatura.domain.GetItemCardUserCase
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.articulo.model.ItemCardVO
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class ItemCardViewModel : BaseViewModel() {
    private val getItemCardUserCase:GetItemCardUserCase = GetItemCardUserCase()
    val version : String = "5.0.0";

    private val _itemcard by lazy { MutableLiveData<ItemCardVO>()  }
    val itemcard: LiveData<ItemCardVO>
        get() = _itemcard

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response


    fun getItemCard(itemFk:String,warehouseFk:String,user:String,password:String) {
        getItemCardUserCase.getItemCard(user,password,itemFk,warehouseFk).enqueue(object :Callback<ItemCardVO>{
            override fun onFailure(call: Call<ItemCardVO>, t: Throwable) {
                _itemcard.value = ItemCardVO(isError = true,errorMessage = "Error al OBTENER "+itemFk+ " Respuesta:"+t.message!!)
            }

            override fun onResponse(call: Call<ItemCardVO>, response: Response<ItemCardVO>) {
                if (response.body() != null){
                    _itemcard.value = response.body()!!
                }else{
                    _itemcard.value = ItemCardVO(isError = true,errorMessage = "Error al llamar a vn.itemCard")
                }
            }

        })
    }

    fun itemStockUpdate(itemFk:String,warehouseFk:String,user:String,password:String,newValue:String,isTrash:String){
        getItemCardUserCase.itemStockUpdate(user,password,itemFk,warehouseFk,newValue,isTrash).enqueue(object : Callback<String>{
            override fun onFailure(call: Call<String>, t: Throwable) {
                _response.value = ResponseItemVO(isError = true,errorMessage = "Error al guardar STOCK "+itemFk+ " Respuesta:"+t.message!!)
            }

            override fun onResponse(call: Call<String>, response: Response<String>) {
                if (response.body() == null){
                    _response.value = ResponseItemVO(isError = true,errorMessage = "Error en la llamada itemStockUpdate")
                }else{
                    _response.value = ResponseItemVO(isError = false,response = response.body()!!)
                }

            }

        })
    }

    fun itemPlacementSave(itemFk:String,warehouseFk:String,user:String,password:String,value:String){
        getItemCardUserCase.itemPlacementSave(user,password,itemFk,warehouseFk,value).enqueue(object : Callback<String>{
            override fun onFailure(call: Call<String>, t: Throwable) {
                _response.value = ResponseItemVO(isError = true,errorMessage ="Error al guardar NICHO "+itemFk+ " Respuesta:"+t.message!!)
            }

            override fun onResponse(call: Call<String>, response: Response<String>) {
                if (response.body() == null){
                    _response.value = ResponseItemVO(isError = true,errorMessage = "Error en la llamada itemPlacementSave")
                }else{
                    _response.value = ResponseItemVO(isError = false,response = response.body()!!)
                }

            }

        })
    }

    fun updateGrouping(itemFk:String,user:String,password:String,value:String,warehouseFk: String){
        getItemCardUserCase.updateGrouping(user,password,itemFk,value,warehouseFk).enqueue(object : Callback<String>{
            override fun onFailure(call: Call<String>, t: Throwable) {
                _response.value = ResponseItemVO(isError = true,errorMessage = "Error al guardar GROUPING "+itemFk+ " Respuesta:"+t.message!!)
            }

            override fun onResponse(call: Call<String>, response: Response<String>) {
                if (response.body() == null){
                    _response.value = ResponseItemVO(isError = true,errorMessage = "Error en la llamada updateGrouping")
                }else{
                    _response.value = ResponseItemVO(isError = false,response = response.body()!!)
                }

            }

        })
    }

    fun updatePacking(itemFk:String,user:String,password:String,value:String,warehouseFk: String){
        getItemCardUserCase.updatePacking(user,password,itemFk,value,warehouseFk).enqueue(object : Callback<String>{
            override fun onFailure(call: Call<String>, t: Throwable) {
                _response.value = ResponseItemVO(isError = true,errorMessage = "Error al guardar PACKING "+itemFk+ " Respuesta:"+t.message!!)
            }

            override fun onResponse(call: Call<String>, response: Response<String>) {
                if (response.body() == null){
                    _response.value = ResponseItemVO(isError = true,errorMessage = "Error en la llamada updatePacking")
                }else{
                    _response.value = ResponseItemVO(isError = false,response = response.body()!!)
                }

            }

        })
    }

    fun itemSaveMin(itemFk:String,user:String,password:String,value:String){
        getItemCardUserCase.itemSaveMin(user,password,itemFk,value).enqueue(object : Callback<String>{
            override fun onFailure(call: Call<String>, t: Throwable) {
                _response.value = ResponseItemVO(isError = true,errorMessage = "Error al guardar MIN "+itemFk+ " Respuesta:"+t.message!!)
            }

            override fun onResponse(call: Call<String>, response: Response<String>) {
                if (response.body() == null){
                    _response.value = ResponseItemVO(isError = true,errorMessage = "Error en la llamada updatePacking")
                }else{
                    _response.value = ResponseItemVO(isError = false,response = response.body()!!)
                }

            }

        })
    }

    fun barcodesEdit(itemFk:String,user:String,password:String,value:String,delete:String){
        getItemCardUserCase.barcodes_edit(user,password,itemFk,value,delete).enqueue(object : Callback<String>{
            override fun onFailure(call: Call<String>, t: Throwable) {
                _response.value = ResponseItemVO(isError = true,errorMessage = "Error al guardar BARCODES "+itemFk+ " Respuesta:"+t.message!!)
            }

            override fun onResponse(call: Call<String>, response: Response<String>) {
                if (response.body() == null){
                    _response.value = ResponseItemVO(isError = true,errorMessage = "Error en la llamada itemBarcode_update")
                }else{
                    _response.value = ResponseItemVO(isError = false,response = response.body()!!)
                }

            }

        })
    }
}
