package es.verdnatura.presentation.view.feature.buscaritem.fragment

import android.content.Context
import android.content.SharedPreferences
import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentBuscarItemBinding
import es.verdnatura.domain.notNull
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnLocationRowClickListener
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.buscaritem.adapter.LocationAdapter
import es.verdnatura.presentation.view.feature.buscaritem.model.ItemLocationVO
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_buscar_item.*
import kotlinx.android.synthetic.main.toolbar.*

class BuscarItemFragment(
    var itemFk:String = ""
) : BaseFragment<FragmentBuscarItemBinding,BuscarItemViewModel>(BuscarItemViewModel::class) {

    private var user = ""
    private var password = ""
    private var warehouseFk = ""
    private var adapter : LocationAdapter? = null
    private lateinit var customDialog: CustomDialog
    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null

    companion object {
        fun newInstance(entryPoint:String) = BuscarItemFragment(entryPoint)
    }

    override fun onAttach(context: Context) {
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }
    override fun getLayoutId(): Int = R.layout.fragment_buscar_item

    override fun init() {
        activity!!.main_bottom_navigation.visibility = View.GONE
        toolbar_title.text = "getItemUbication"
        customDialog = CustomDialog(requireContext())
        setEvents()
        if (!itemFk.isNullOrEmpty()){
            getLocations(itemFk)
        }
        super.init()
    }


    private fun setEvents(){
        edit_itemFk.requestFocus()
        edit_itemFk.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!edit_itemFk.text.toString().isNullOrEmpty())
                    getLocations(edit_itemFk.text.toString())
                edit_itemFk.setText("")
                (activity as MainActivity).hideKeyboard(edit_itemFk)
                return@setOnEditorActionListener true
            }
            false
        }


        backButton.setOnClickListener {
            activity!!.onBackPressed()
        }

    }

    private fun getLocations(itemFk:String){
        val prefs: SharedPreferences = activity!!.getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        this.itemFk = itemFk
        splash_progress.visibility = View.VISIBLE
        viewModel.getLocations(user,password,itemFk)
    }

    override fun observeViewModel() {
        with(viewModel){
            loadLocationList.observe(viewLifecycleOwner, Observer { event ->

                event.getContentIfNotHandled().notNull {
                    splash_progress.visibility = View.GONE
                    adapter = LocationAdapter(it.list,pasillerosItemClickListener!!)

                    location_recyclerview.adapter = adapter
                    location_recyclerview.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

                    var totalVisible:Int = 0
                    it.list.forEach {
                        if (!it.visible.isNullOrEmpty())
                            totalVisible += it.visible.toInt()
                    }
                    toolbar_title.text = "Item: "+itemFk+ " Total visible: "+totalVisible


                }
            })
        }
    }

}
