package es.verdnatura.presentation.view.feature.faltas.fragment

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations
import es.verdnatura.domain.GetInventaryUserCase
import es.verdnatura.domain.GetItemCardUserCase
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.faltas.model.ItemFaltasListVO
import es.verdnatura.presentation.view.feature.faltas.model.ItemFaltasVO
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class FaltasViewModel : BaseViewModel() {


    private val getInventaryUserCase: GetInventaryUserCase = GetInventaryUserCase()
    private val getItemCardUserCase: GetItemCardUserCase = GetItemCardUserCase()


    private val _faltasList by lazy { MutableLiveData<ItemFaltasListVO>() }
    val faltasList: LiveData<ItemFaltasListVO>
        get() = _faltasList

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response


    val loadFaltasList = Transformations.map(_faltasList) { Event(it) }


    fun getFaltas(user:String,password:String,warehouseFk:String){
        getInventaryUserCase.faultsReview(user,password,warehouseFk).enqueue(object :
            Callback<List<ItemFaltasVO>> {
            override fun onFailure(call: Call<List<ItemFaltasVO>>, t: Throwable) {
                val listError:ArrayList<ItemFaltasVO> = ArrayList()
                listError.add(ItemFaltasVO(isError = true,errorMessage = t.message!!))
                _faltasList.value = ItemFaltasListVO(listError)
            }

            override fun onResponse(
                call: Call<List<ItemFaltasVO>>,
                response: Response<List<ItemFaltasVO>>
            ) {
                if (response.body() != null){
                    _faltasList.value = response.body()?.let { ItemFaltasListVO(it) }
                }else{
                    val listError:ArrayList<ItemFaltasVO> = ArrayList()
                    listError.add(ItemFaltasVO(isError = true,errorMessage = "Error en la llamada de faultsReview"))
                    _faltasList.value = ItemFaltasListVO(listError)
                }
            }

        })
    }

    fun itemStockUpdate(itemFk:String,warehouseFk:String,user:String,password:String,newValue:String,isTrash:String){
        getItemCardUserCase.itemStockUpdate(user,password,itemFk,warehouseFk,newValue,isTrash).enqueue(object :
            Callback<String> {
            override fun onFailure(call: Call<String>, t: Throwable) {
                _response.value = ResponseItemVO(isError = true,errorMessage = "Error al guardar STOCK "+itemFk+ " Respuesta:"+t.message!!)
            }

            override fun onResponse(call: Call<String>, response: Response<String>) {
                if (response.body() == null){
                    _response.value = ResponseItemVO(isError = true,errorMessage = "Error en la llamada itemStockUpdate")
                }else{
                    _response.value = ResponseItemVO(isError = false,response = response.body()!!)
                }

            }

        })
    }

    fun faultsReview_isChecked(user:String,password:String,itemFk:String,warehouseFk:String){
        getInventaryUserCase.faultsReview_isChecked(user,password,itemFk,warehouseFk).enqueue(object :
            Callback<String> {
            override fun onFailure(call: Call<String>, t: Throwable) {
                _response.value = ResponseItemVO(isError = true,errorMessage = "Error faultsReview_isChecked "+itemFk+ " Respuesta:"+t.message!!)
            }
            override fun onResponse(call: Call<String>, response: Response<String>) {
                if (response.body() == null){
                    _response.value = ResponseItemVO(isError = true,errorMessage = "Error en la llamada faultsReview_isChecked")
                }else{
                    _response.value = ResponseItemVO(isError = false,response = response.body()!!)
                }

            }
        })
    }

}
