package es.verdnatura.presentation.view.feature.inventario.fragment

import android.content.Context
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.view.View
import android.view.inputmethod.EditorInfo
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentInventaryBinding
import es.verdnatura.domain.notNull
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnInvetoryNichoClickListener
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.feature.inventario.adapter.InventoryAdapter
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.inventario.model.ItemInventaryVO
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_buscar_item.location_recyclerview
import kotlinx.android.synthetic.main.fragment_buscar_item.splash_progress
import kotlinx.android.synthetic.main.fragment_inventary.*
import kotlinx.android.synthetic.main.toolbar.*

class InventaryFragment : BaseFragment<FragmentInventaryBinding,InventaryViewModel>(InventaryViewModel::class) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var adapter : InventoryAdapter? = null
    private lateinit var customDialogInput: CustomDialogInput
    private var listInvetory:ArrayList<ItemInventaryVO> = ArrayList()
    private var listInvetoryAux:ArrayList<ItemInventaryVO> = ArrayList()
    private lateinit var customDialog: CustomDialog
    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null

    companion object {
        fun newInstance() = InventaryFragment()
    }

    override fun getLayoutId(): Int = R.layout.fragment_inventary

    override fun onAttach(context: Context) {
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }

    override fun init() {

        val prefs: SharedPreferences = activity!!.getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        sectorFk = prefs.getInt(SECTORFK,1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        viewModel.getInventory(user,password,sectorFk)
        customDialogInput = CustomDialogInput(requireContext())
        customDialog = CustomDialog(requireContext())

        activity!!.main_bottom_navigation.visibility = View.GONE
        splash_progress.visibility = View.VISIBLE
        toolbar_title.text = "itemShelvingRadar"
        setToolBar()
        setEvents()

        super.init()
    }




    private fun setToolBar(){
        val listIcons:ArrayList<Drawable> = ArrayList()
        val iconReload : Drawable = resources.getDrawable(R.drawable.ic_autorenew_black_24dp,resources.newTheme())
        listIcons.add(iconReload)
        toolbar_icons.adapter = ToolBarAdapter(listIcons,object: OnOptionsSelectedListener{
            override fun onOptionsItemSelected(item: Drawable) {
               if (item == iconReload){
                   splash_progress.visibility = View.VISIBLE
                   viewModel.getInventory(user,password,sectorFk)
               }
            }

        })
        toolbar_icons.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun setEvents(){
        backButton.setOnClickListener {
            activity!!.onBackPressed()
        }

        filter_itemFk.setOnKeyListener { v, keyCode, event ->
            if (filter_itemFk.text.toString().isNullOrEmpty()){
                if (listInvetory.size != listInvetoryAux.size){
                    listInvetory.removeAll(listInvetoryAux)
                    listInvetoryAux.forEach {
                        listInvetory.add(it)
                    }

                }

            }else{
                listInvetory.removeAll(listInvetoryAux)
                listInvetoryAux.forEach {
                    if (it.itemFk.contains(filter_itemFk.text.toString(),true)){
                        listInvetory.add(it)
                    }
                }


            }
            adapter!!.notifyDataSetChanged()
            return@setOnKeyListener false
        }
    }

    override fun observeViewModel() {
        with(viewModel){
            loadInventaryList.observe(viewLifecycleOwner, Observer { event ->

                event.getContentIfNotHandled().notNull {
                    splash_progress.visibility = View.GONE
                    listInvetory = ArrayList()
                    listInvetoryAux = ArrayList()
                    it.list.forEach {
                        if (it.nicho != "0"){
                            listInvetory.add(it)
                            listInvetoryAux.add(it)
                        }
                    }

                    adapter = InventoryAdapter(listInvetory,object: OnInvetoryNichoClickListener{
                        override fun onInvetoryNichoClickListener(item: ItemInventaryVO) {
                            customDialogInput.setTitle(item.itemFk+"\n"+item.longName+" "+item.size).setDescription("Cantidad real("+item.nicho+")").setOkButton("Tirar"){
                                viewModel.itemStockUpdate(item.itemFk,warehouseFk,user,password,customDialogInput.getValue(),"0")
                                changeOfflineValue(item)
                                customDialogInput.hide()
                            }.setKoButton("Cancelar"){
                                customDialogInput.hide()
                            }.setValue("0").show()

                            customDialogInput.getEditText().requestFocus()
                            customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
                                if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                                    if (!customDialogInput.getValue().isNullOrEmpty()){
                                        viewModel.itemStockUpdate(item.itemFk,warehouseFk,user,password,customDialogInput.getValue(),"0")
                                        changeOfflineValue(item)
                                    }
                                    customDialogInput.setValue("")
                                    customDialogInput.hide()
                                    (activity as MainActivity).hideKeyboard(customDialogInput.getEditText())
                                    return@setOnEditorActionListener true
                                }
                                false
                            }

                        }
                    },onPasillerosItemClickListener = pasillerosItemClickListener!!)
                    location_recyclerview.adapter = adapter
                    location_recyclerview.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
                }
            })


            response.observe(viewLifecycleOwner, Observer {
                if (it.isError){
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.hide()
                    }.show()
                }
            })
        }
    }

    private fun changeOfflineValue(item:ItemInventaryVO){
        var i = 0
        var position = 0
        listInvetory.forEach {
            if (it.itemFk == item.itemFk) {
                position = i
            }
            i = i.plus(1)
        }
        listInvetory.removeAt(position)

        i = 0
        position = 0
        listInvetoryAux.forEach {
            if (it.itemFk == item.itemFk) {
                position = i
            }
            i = i.plus(1)
        }
        listInvetoryAux.removeAt(position)
        adapter?.notifyDataSetChanged()
    }


}
