package es.verdnatura.presentation.view.feature.inventario.fragment

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations
import es.verdnatura.domain.GetInventaryUserCase
import es.verdnatura.domain.GetItemCardUserCase
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.inventario.model.InventaryListVO
import es.verdnatura.presentation.view.feature.inventario.model.ItemInventaryVO
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class InventaryViewModel : BaseViewModel() {


    private val getInventaryUserCase: GetInventaryUserCase = GetInventaryUserCase()
    private val getItemCardUserCase: GetItemCardUserCase = GetItemCardUserCase()


    private val _inventaryList by lazy { MutableLiveData<InventaryListVO>() }
    val inventaryList: LiveData<InventaryListVO>
        get() = _inventaryList

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response


    val loadInventaryList = Transformations.map(_inventaryList) { Event(it) }


    fun getInventory(user:String,password:String,sectorFk:String){
        getInventaryUserCase.itemShelvingRadar(user,password,sectorFk).enqueue(object :
            Callback<List<ItemInventaryVO>> {
            override fun onFailure(call: Call<List<ItemInventaryVO>>, t: Throwable) {
                val listError:ArrayList<ItemInventaryVO> = ArrayList()
                listError.add(ItemInventaryVO(isError = true,errorMessage = t.message!!))
                _inventaryList.value = InventaryListVO(listError)
            }

            override fun onResponse(
                call: Call<List<ItemInventaryVO>>,
                response: Response<List<ItemInventaryVO>>
            ) {
                if (response.body() != null){
                    _inventaryList.value = response.body()?.let { InventaryListVO(it) }
                }else{
                    val listError:ArrayList<ItemInventaryVO> = ArrayList()
                    listError.add(ItemInventaryVO(isError = true,errorMessage = "Error en la llamada de itemShelvingRadar"))
                    _inventaryList.value = InventaryListVO(listError)
                }
            }

        })
    }

    fun itemStockUpdate(itemFk:String,warehouseFk:String,user:String,password:String,newValue:String,isTrash:String){
        getItemCardUserCase.itemStockUpdate(user,password,itemFk,warehouseFk,newValue,isTrash).enqueue(object : Callback<String>{
            override fun onFailure(call: Call<String>, t: Throwable) {
                _response.value = ResponseItemVO(isError = true,errorMessage = "Error al guardar STOCK "+itemFk+ " Respuesta:"+t.message!!)
            }

            override fun onResponse(call: Call<String>, response: Response<String>) {
                if (response.body() == null){
                    _response.value = ResponseItemVO(isError = true,errorMessage = "Error en la llamada itemStockUpdate")
                }else{
                    _response.value = ResponseItemVO(isError = false,response = response.body()!!)
                }

            }

        })
    }

}
