package es.verdnatura.presentation.view.feature.login.fragment


import android.content.Intent
import android.content.SharedPreferences
import android.net.Uri
import android.view.View
import androidx.lifecycle.Observer
import es.verdnatura.R
import es.verdnatura.databinding.FragmentLoginBinding
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.login.model.LoginItemVO
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import kotlinx.android.synthetic.main.fragment_login.*

class LoginFragment : BaseFragment<FragmentLoginBinding,LoginViewModel> (LoginViewModel::class) {

    private lateinit var customDialog: CustomDialog
    companion object {
        fun newInstance() = LoginFragment()
    }

    override fun getLayoutId(): Int = R.layout.fragment_login

    override fun init() {
        customDialog = CustomDialog(requireContext())
        checkUser()
        button_login.setOnClickListener(View.OnClickListener {
            splash_progress.visibility = View.VISIBLE
            viewModel.login(edittext_username.text.toString(),edittext_password.text.toString())
        })
    }

    private fun checkUser(){
        val prefs: SharedPreferences = activity!!.getSharedPreferences(PREFS_USER,0)
        edittext_username.setText(prefs.getString(USER,""))
        edittext_password.setText(prefs.getString(PASSWORD,""))
        if (edittext_password.text.toString().isNotEmpty()){
            switch_remember.isChecked = true
        }
    }

    override fun observeViewModel() {
        with(viewModel){
            loginitem.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.INVISIBLE
                if (it.isError){
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.hide()
                    }.show()
                }else{
                    splash_progress.visibility = View.VISIBLE
                    saveUserFkPref(it)
                    if (switch_remember.isChecked) {
                        saveUserAccesPref(edittext_username.text.toString(),edittext_password.text.toString())
                    }else{
                        saveUserAccesPref("","")
                    }
                    loginSalix(user = edittext_username.text.toString(),password = edittext_password.text.toString())
                }

            })
            loginsalixitem.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.INVISIBLE
                if (it.isError){
                    saveTokenPref("")
                    customDialog.setTitle("Error").setDescription(it.errorMessage+". Puedes continuar pero algunas funcionalidades no estarán disponibles.").setOkButton("Entendido"){
                        customDialog.hide()
                        getVersion()
                    }.show()
                }else{
                    saveTokenPref(it.token)
                    getVersion()
                }
            })

            version.observe(viewLifecycleOwner, Observer {
                splash_progress.visibility = View.INVISIBLE
                if (it){
                    goToMain()
                }else{
                    customDialog.setTitle("Error").setDescription("Existe una versión nueva, es recomendable actualizar.").setOkButton("Actualizar"){
                        val openURL = Intent(Intent.ACTION_VIEW)
                        openURL.data = Uri.parse("https://app.verdnatura.es/bin/vn-pickingV2.apk")
                        startActivity(openURL)
                    }.setKoButton("Omitir"){
                        customDialog.hide()
                        goToMain()
                    }.show()
                }
            })
        }
        super.observeViewModel()
    }

    private fun saveTokenPref(token:String){
        val prefs: SharedPreferences = activity!!.getSharedPreferences(PREFS_USER,0)
        val editor = prefs.edit()
        editor.putString(TOKEN,token)
        editor.apply()
    }

    private fun saveUserFkPref(loginitem : LoginItemVO){
        val prefs: SharedPreferences = activity!!.getSharedPreferences(PREFS_USER,0)
        val editor = prefs.edit()
        editor.putString(USERFK,loginitem.id)
        editor.apply()
    }

    private fun saveUserAccesPref(user:String,password:String){
        val prefs: SharedPreferences = activity!!.getSharedPreferences(PREFS_USER,0)
        val editor = prefs.edit()
        editor.putString(USER,user)
        editor.putString(PASSWORD,password)
        editor.apply()
    }

    private fun goToMain(){
        val intent = Intent(activity,MainActivity::class.java)
        startActivity(intent)
    }

    private fun getVersion(){
        val versionName = activity!!.packageManager.getPackageInfo(activity!!.packageName,0).versionName
        splash_progress.visibility = View.VISIBLE
        viewModel.checkVersion(user = edittext_username.text.toString(),password = edittext_password.text.toString(),version = versionName)
    }

}
