package es.verdnatura.presentation.view.feature.login.fragment

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import es.verdnatura.domain.GetLoginUserCase
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.view.feature.login.model.LoginItemVO
import es.verdnatura.presentation.view.feature.login.model.LoginSalixVO
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class LoginViewModel() : BaseViewModel() {

    private val getLoginUserCase: GetLoginUserCase = GetLoginUserCase()


    private val _loginitem by lazy { MutableLiveData<LoginItemVO> ()}
    val loginitem : LiveData<LoginItemVO>
        get() = _loginitem

    private val _loginsalixitem by lazy { MutableLiveData<LoginSalixVO> ()}
    val loginsalixitem : LiveData<LoginSalixVO>
        get() = _loginsalixitem


    private val _version by lazy { MutableLiveData<Boolean> ()}
    val version : LiveData<Boolean>
        get() = _version


    fun login(user:String,password:String){
        getLoginUserCase.login(user,password).enqueue(object : Callback<String>{
            override fun onResponse(call: Call<String>, response: Response<String>) {

               var loginItemVO:LoginItemVO?
                if (response.raw().code() == 555){
                     loginItemVO = LoginItemVO(id = "0",isError = true, errorMessage = "Usuario o contraseña incorrectos")
                }else if (!response.body().isNullOrEmpty()){
                     loginItemVO  = response.body()?.let { LoginItemVO(id = it) }
                }else{
                    loginItemVO = LoginItemVO(id = "0",isError = true,errorMessage = "Error desconocido")
                }

                _loginitem.value = loginItemVO!!
            }

            override fun onFailure(call: Call<String>, t: Throwable) {
                val loginItemVO : LoginItemVO? =  LoginItemVO(id = "0",isError = true, errorMessage = t.message!!)
                _loginitem.value = loginItemVO
            }
        })
    }

    fun loginSalix(user:String,password:String){
        val loginSalixVO = LoginSalixVO(user,password, "aaa")
        _loginsalixitem.value = loginSalixVO
        /*
        getLoginUserCase.salixLogin(user,password).enqueue(object : Callback<LoginSalixVO>{
            override fun onResponse(call: Call<LoginSalixVO>, response: Response<LoginSalixVO>) {

                var loginSalixVO:LoginSalixVO?
                if(response.body() != null){
                    loginSalixVO = response.body()?.token?.let { LoginSalixVO(user,password, it,isError = false,errorMessage = "") }
                }else{
                    loginSalixVO = LoginSalixVO(user,password,isError = true,errorMessage = "Error al conectar con salix")
                }

                _loginsalixitem.value = loginSalixVO
            }

            override fun onFailure(call: Call<LoginSalixVO>, t: Throwable) {
                val loginSalixVO = LoginSalixVO(user,password, "",isError = true,errorMessage = t.message!!)
                _loginsalixitem.value = loginSalixVO
            }
        })*/
    }

    fun checkVersion(user:String,password:String,version:String){
        getLoginUserCase.checkVersion(user,password,version).enqueue(object : Callback<Boolean>{
            override fun onResponse(call: Call<Boolean>, response: Response<Boolean>) {
                if (response.body() != null){
                    _version.value = response.body()
                }else{
                    _version.value = true
                }
            }

            override fun onFailure(call: Call<Boolean>, t: Throwable) {
                _version.value = true
            }
        })
    }

}
