package es.verdnatura.presentation.view.feature.main.activity

import android.content.SharedPreferences
import android.util.Log
import android.view.Menu
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import es.verdnatura.R
import es.verdnatura.databinding.ActivityMainBinding
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseActivity
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.common.TAG
import es.verdnatura.presentation.common.addFragment
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.ajustes.fragment.AjustesFragment
import es.verdnatura.presentation.view.feature.articulo.fragment.ItemCardFragment
import es.verdnatura.presentation.view.feature.buscaritem.fragment.BuscarItemFragment
import es.verdnatura.presentation.view.feature.faltas.fragment.FaltasFragment
import es.verdnatura.presentation.view.feature.inventario.fragment.InventaryFragment
import es.verdnatura.presentation.view.feature.main.model.ItemMenuVO
import es.verdnatura.presentation.view.feature.pasillero.fragment.PasilleroFragment
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import es.verdnatura.presentation.view.feature.shelvingparking.fragment.ShelvingParkingFragment
import es.verdnatura.presentation.view.feature.ubicador.fragment.UbicadorFragment
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch


class MainActivity : BaseActivity<ActivityMainBinding>() , OnPasillerosItemClickListener {

    private var lastBottomMenuItemSelected: ItemMenuVO? = null
    private lateinit var customDialog: CustomDialog
    var firstItem : ItemMenuVO? = null
    var fm = supportFragmentManager

    override fun getLayoutId(): Int = R.layout.activity_main


    override fun init() {
        customDialog = CustomDialog(this)
        setBottomMenu()
        if(haveSector()) addFragment(PasilleroFragment.newInstance(),R.id.main_frame_layout, PasilleroFragment.TAG,false)
        else addFragment(AjustesFragment.newInstance(),R.id.main_frame_layout, AjustesFragment.TAG,false)
    }


    private fun haveSector() : Boolean{
        val prefs: SharedPreferences = getSharedPreferences("es.verdnatura.user.prefs",0)
        val sectorFk = prefs.getInt("sectorFk",-1)
        return sectorFk != -1
    }

    private fun setBottomMenu(){
        val bottomMenu = main_bottom_navigation
        bottomMenu.itemIconTintList = null
        GlobalScope.launch(Dispatchers.Main){
            val bottomMenuItems : ArrayList<ItemMenuVO> = ArrayList()

            bottomMenuItems.add(ItemMenuVO(0,title = "Pasilleros",defaultImage = ContextCompat.getDrawable(
                applicationContext,R.drawable.ic_device_hub_black_24dp),selectedImage = ContextCompat.getDrawable(
                applicationContext,R.drawable.ic_device_hub_black_24dp_selected)))

            bottomMenuItems.add(ItemMenuVO(1,title = "Sacadores",defaultImage = ContextCompat.getDrawable(
                applicationContext,R.drawable.ic_local_florist_black_24dp),selectedImage = ContextCompat.getDrawable(
                applicationContext,R.drawable.ic_local_florist_black_24dp_selected)))

            bottomMenuItems.add(ItemMenuVO(2,title = "Controladores",defaultImage = ContextCompat.getDrawable(
                applicationContext,R.drawable.ic_verified_user_black_24dp),selectedImage = ContextCompat.getDrawable(
                applicationContext,R.drawable.ic_verified_user_black_24dp_selected)))

            bottomMenuItems.add(ItemMenuVO(3,title = "Ajustes",defaultImage = ContextCompat.getDrawable(
                applicationContext,R.drawable.ic_build_black_24dp),selectedImage = ContextCompat.getDrawable(
                applicationContext,R.drawable.ic_build_black_24dp_selected)))

            // Add al items to menu view
            bottomMenuItems.forEach {
                bottomMenu.menu.add(Menu.NONE, it.id, Menu.NONE, it.title).icon = it.defaultImage
            }


            if(haveSector()){
                // Select first item by default
                firstItem = bottomMenuItems.first()
                bottomMenu.selectedItemId = firstItem!!.id
                bottomMenu.menu.findItem(firstItem!!.id).icon = firstItem!!.selectedImage
                lastBottomMenuItemSelected = firstItem
            }else{
                // Select last item by default
                val lastItem = bottomMenuItems.last()
                bottomMenu.selectedItemId = lastItem.id
                bottomMenu.menu.findItem(lastItem.id).icon = lastItem.selectedImage
                lastBottomMenuItemSelected = lastItem
            }


            bottomMenu.setOnNavigationItemSelectedListener { menuItem ->
                // Get selected item from itemList and apply ui changes to menu view
                val selectedItemMenu = bottomMenuItems.find { menuItem.itemId == it.id }
                menuItem.icon = selectedItemMenu?.selectedImage

                // Get last selected item and disable it
                bottomMenu.menu.findItem(lastBottomMenuItemSelected?.id!!).icon =
                    lastBottomMenuItemSelected?.defaultImage
                lastBottomMenuItemSelected = selectedItemMenu


                when (selectedItemMenu?.title) {
                    "Pasilleros" -> {
                        fm.popBackStack(null,FragmentManager.POP_BACK_STACK_INCLUSIVE)
                        addFragment(PasilleroFragment.newInstance(),R.id.main_frame_layout, PasilleroFragment.TAG,false)
                        true
                    }
                    "Sacadores" -> {
                        selectedItemMenu.title.toast(this@MainActivity)
                        true
                    }
                    "Controladores" -> {
                        selectedItemMenu.title.toast(this@MainActivity)
                        true
                    }
                    "Ajustes" -> {
                        fm.popBackStack(null,FragmentManager.POP_BACK_STACK_INCLUSIVE)
                        addFragment(AjustesFragment.newInstance(),R.id.main_frame_layout, AjustesFragment.TAG,false)
                    }


                    else -> false
                }

                true
            }
            bottomMenu!!.setOnNavigationItemReselectedListener {

            }
        }
    }

    override fun onPasillerosItemClickListener(item: PasillerosItemVO, entryPoint: String) {
        when (item.title){
            "Consultar artículo" -> {
                addFragmentOnTop(ItemCardFragment.newInstance(entryPoint))
            }
            "Buscar item" -> {
                addFragmentOnTop(BuscarItemFragment.newInstance(entryPoint))
            }
            "Inventario" -> {
                addFragmentOnTop(InventaryFragment.newInstance())
            }
            "Faltas" -> {
                addFragmentOnTop(FaltasFragment.newInstance())
            }
            "Shelving Parking" -> {
                addFragmentOnTop(ShelvingParkingFragment.newInstance())
            }
            "Ubicador" -> {
                addFragmentOnTop(UbicadorFragment.newInstance(entryPoint))
            }
        }
        Log.i("Item: ",item.title)
    }

    fun addFragmentOnTop(fragment: Fragment?) {
        supportFragmentManager
            .beginTransaction()
            .replace(R.id.main_frame_layout, fragment!!)
            .addToBackStack(null)
            .commit()
    }



    override fun onBackPressed() {

        if (fm.backStackEntryCount > 0) {
            fm.popBackStackImmediate()
        }else {
            customDialog.setTitle("Cerrar sesión").setDescription("¿Estás seguro de cerrar la sesión?").setOkButton("Salir"){
                customDialog.hide()
                finish()
            }.setKoButton("Cancelar") {
                customDialog.hide()
            }.show()
        }

    }

}
