package es.verdnatura.presentation.view.feature.ubicador.fragment

import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.view.View
import android.view.inputmethod.EditorInfo
import android.widget.Toast
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentUbicadorBinding
import es.verdnatura.domain.notNull
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.component.CustomDialogUbicador
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.inventario.model.ItemInventaryVO
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.shelvingparking.adapter.ShelvingParkingAdapter
import es.verdnatura.presentation.view.feature.ubicador.adapter.UbicadorAdapter
import es.verdnatura.presentation.view.feature.ubicador.model.ItemUbicadorVO

import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.fragment_buscar_item.*
import kotlinx.android.synthetic.main.fragment_ubicador.*
import kotlinx.android.synthetic.main.fragment_ubicador.location_recyclerview
import kotlinx.android.synthetic.main.fragment_ubicador.splash_progress
import kotlinx.android.synthetic.main.toolbar.*

class UbicadorFragment(
    var shelvingFk:String = ""
) : BaseFragment<FragmentUbicadorBinding, UbicadorViewModel>(
    UbicadorViewModel::class) {

    private var user = ""
    private var password = ""
    private var sectorFk = ""
    private var warehouseFk = ""
    private var adapter : UbicadorAdapter? = null
    private lateinit var customDialogInput: CustomDialogInput
    private lateinit var customDialogUbicador : CustomDialogUbicador
    private var listItems:ArrayList<ItemUbicadorVO> = ArrayList()
    private lateinit var customDialog: CustomDialog
    private var deep:Int = 1


    companion object {
        fun newInstance(entryPoint:String) = UbicadorFragment(entryPoint)
    }

    override fun getLayoutId(): Int = R.layout.fragment_ubicador


    override fun init() {

        val prefs: SharedPreferences = activity!!.getSharedPreferences(PREFS_USER,0)
        user = prefs.getString(USER,"").toString()
        password = prefs.getString(PASSWORD,"").toString()
        sectorFk = prefs.getInt(SECTORFK,1).toString()
        warehouseFk = prefs.getInt(WAREHOUSEFK,1).toString()
        customDialogInput = CustomDialogInput(requireContext())
        customDialog = CustomDialog(requireContext())
        customDialogUbicador = CustomDialogUbicador(requireContext())
        activity!!.main_bottom_navigation.visibility = View.GONE
        toolbar_title.text = "itemShelvingList"
        setEvents()
        setToolBar()
        if (!shelvingFk.isNullOrEmpty()){
            splash_progress.visibility = View.VISIBLE
            toolbar_title.text = shelvingFk
            viewModel.itemShelvingList(user,password,shelvingFk)
        }
        super.init()
    }

    private fun setToolBar(){
        val listIcons:ArrayList<Drawable> = ArrayList()
        val iconAdd : Drawable = resources.getDrawable(R.drawable.ic_add_black_24dp,resources.newTheme())
        val iconReload : Drawable = resources.getDrawable(R.drawable.ic_autorenew_black_24dp,resources.newTheme())
        listIcons.add(iconAdd)
        listIcons.add(iconReload)
        toolbar_icons.adapter = ToolBarAdapter(listIcons,object: OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {
                if (item == iconAdd){
                    showAddItem()
                }else if(item == iconReload){
                    splash_progress.visibility = View.VISIBLE
                    viewModel.itemShelvingList(user,password,shelvingFk)
                }
            }

        })
        toolbar_icons.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun setEvents(){
        backButton.setOnClickListener {
            activity!!.onBackPressed()
        }

        edit_matricula.requestFocus()
        edit_matricula.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!edit_matricula.text.toString().isNullOrEmpty()){
                    toolbar_title.text = edit_matricula.text.toString()
                    shelvingFk = edit_matricula.text.toString()
                    splash_progress.visibility = View.VISIBLE
                    viewModel.itemShelvingList(user,password,edit_matricula.text.toString())
                }

                edit_matricula.setText("")
                (activity as MainActivity).hideKeyboard(edit_matricula)
                return@setOnEditorActionListener true
            }

            return@setOnEditorActionListener false
        }


    }

    override fun observeViewModel() {
        with(viewModel){
            loadShelvingList.observe(viewLifecycleOwner, Observer { event ->
                event.getContentIfNotHandled().notNull {
                    deep = it.list.size
                    listItems = ArrayList()
                    it.list.forEach { item ->
                        listItems.add(item)
                    }
                    splash_progress.visibility = View.GONE
                    adapter = UbicadorAdapter(listItems)
                    location_recyclerview.adapter = adapter
                    location_recyclerview.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

                }
            })

            response.observe(viewLifecycleOwner, Observer {
                if (it.isError){
                    customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.hide()
                    }.show()
                }
            })
        }
    }

    private fun showAddItem(){
        customDialogUbicador.setTitle("Nuevo item").setItemValue("").setPackingValue("").setEtiquetaValue("").setVisibleValue("").setOkButton("Guardar"){
            saveNewItem(customDialogUbicador.getItemValue(),customDialogUbicador.getEtiquetaValue(),customDialogUbicador.getPackingValue(),customDialogUbicador.getVisibleValue())
        }.setKoButton("Cerrar"){
            customDialogUbicador.hide()
        }.show()

        customDialogUbicador.getEditItem().requestFocus()
        (activity as MainActivity).hideKeyboard(customDialogUbicador.getEditItem())

        customDialogUbicador.getVisibleItem().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                saveNewItem(customDialogUbicador.getItemValue(),customDialogUbicador.getEtiquetaValue(),customDialogUbicador.getPackingValue(),customDialogUbicador.getVisibleValue())
                return@setOnEditorActionListener true
            }
            false
        }

        customDialogUbicador.getEtiquetaItem().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                saveNewItem(customDialogUbicador.getItemValue(),customDialogUbicador.getEtiquetaValue(),customDialogUbicador.getPackingValue(),customDialogUbicador.getVisibleValue())
                return@setOnEditorActionListener true
            }
            false
        }


    }

    private fun saveNewItem(itemFk:String,etiquetas:String,packing:String,visible:String){
            if (!packing.isNullOrEmpty()){
                if (!itemFk.isNullOrEmpty() && !visible.isNullOrEmpty()){
                    viewModel.itemShelvingMake(user,password,shelvingFk,itemFk,deep.toString(),visible,packing,warehouseFk,"1")
                    changeOfflineValue(ItemUbicadorVO(item = itemFk,visible = visible,packing = packing))
                    customDialogUbicador.setItemValue("").setPackingValue("").setEtiquetaValue("").setVisibleValue("")
                    customDialogUbicador.getEditItem().requestFocus()
                    (activity as MainActivity).hideKeyboard(customDialogUbicador.getEditItem())
                }else if(!itemFk.isNullOrEmpty() && !etiquetas.isNullOrEmpty()){
                    val visible_calc = (etiquetas.toInt() * packing.toInt()).toString()
                    viewModel.itemShelvingMake(user,password,shelvingFk,itemFk,deep.toString(),visible_calc,packing,warehouseFk,"1")
                    changeOfflineValue(ItemUbicadorVO(item = itemFk,visible = visible_calc,packing = packing,stickers = etiquetas))
                    customDialogUbicador.setItemValue("").setPackingValue("").setEtiquetaValue("").setVisibleValue("")
                    customDialogUbicador.getEditItem().requestFocus()
                    (activity as MainActivity).hideKeyboard(customDialogUbicador.getEditItem())
                }else{
                    "Rellena campos obligatorios".toast((activity as MainActivity), Toast.LENGTH_LONG)
                }
            }else{
                "Packing obligatorio".toast((activity as MainActivity), Toast.LENGTH_LONG)
            }
    }

    private fun changeOfflineValue(item:ItemUbicadorVO){
        listItems.add(item)
        adapter?.notifyDataSetChanged()
    }
}