package es.verdnatura.presentation.view.feature.ubicador.fragment

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations
import androidx.lifecycle.ViewModel
import es.verdnatura.domain.GetShelvingParkingUserCase
import es.verdnatura.domain.GetUbicadorUserCase
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.shelvingparking.model.ItemShelvingParkingListVO
import es.verdnatura.presentation.view.feature.shelvingparking.model.ItemShelvingParkingVO
import es.verdnatura.presentation.view.feature.ubicador.model.ItemUbicadorListVO
import es.verdnatura.presentation.view.feature.ubicador.model.ItemUbicadorVO
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class UbicadorViewModel : BaseViewModel() {


    private val getUbicadorUserCase: GetUbicadorUserCase = GetUbicadorUserCase()

    private val _shelvingList by lazy { MutableLiveData<ItemUbicadorListVO>() }
    val shelvingList: LiveData<ItemUbicadorListVO>
        get() = _shelvingList

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response


    val loadShelvingList = Transformations.map(_shelvingList) { Event(it) }


    fun itemShelvingList(usuario:String,password:String,vShelvingFk:String){
        getUbicadorUserCase.itemShelvingList(usuario,password,vShelvingFk.toUpperCase()).enqueue(object :
            Callback<List<ItemUbicadorVO>> {
            override fun onFailure(call: Call<List<ItemUbicadorVO>>, t: Throwable) {
                val listError:ArrayList<ItemUbicadorVO> = ArrayList()
                listError.add(ItemUbicadorVO(isError = true,errorMessage = t.message!!))
                _shelvingList.value = ItemUbicadorListVO(listError)
            }

            override fun onResponse(
                call: Call<List<ItemUbicadorVO>>,
                response: Response<List<ItemUbicadorVO>>
            ) {
                if (response.body() != null){
                    _shelvingList.value = response.body()?.let { ItemUbicadorListVO(it) }
                }else{
                    val listError:ArrayList<ItemUbicadorVO> = ArrayList()
                    listError.add(ItemUbicadorVO(isError = true,errorMessage = "Error en la llamada de itemShelvingList"))
                    _shelvingList.value = ItemUbicadorListVO(listError)
                }
            }

        })
    }


    fun itemShelvingMake(usuario:String,password:String,shelving:String,item:String,deep:String,quantity:String,packing:String,warehouse:String,level:String){
        getUbicadorUserCase.itemShelvingMake(usuario,password,shelving,item,deep,quantity,packing,warehouse,level).enqueue(object : Callback<String>{
            override fun onFailure(call: Call<String>, t: Throwable) {
                _response.value = ResponseItemVO(isError = true,errorMessage ="Error al guardar item: "+item+ " Respuesta:"+t.message!!)
            }

            override fun onResponse(call: Call<String>, response: Response<String>) {
                if (response.body() == null){
                    _response.value = ResponseItemVO(isError = true,errorMessage = "Error en la llamada itemShelvingMake")
                }else{
                    _response.value = ResponseItemVO(isError = false,response = response.body()!!)
                }

            }

        })
    }


}
